//------------------------------------------------------------------
// Copyright (c) 2004-2007 PlayFirst, Inc. All rights reserved.
// Part of the Playground SDK(tm).
//
// This file contains proprietary information of PlayFirst, Inc.
// It is provided subject to a license agreement. It may not be
// reproduced, used, disclosed, sold or transferred without the
// prior written consent of PlayFirst, Inc.
//------------------------------------------------------------------

#ifndef EVENT_H_INCLUDED
#define EVENT_H_INCLUDED

#ifndef POINT_H_INCLUDED
#include "point.h"
#endif
/**
 * @condfile
 * Interface for TEvent, and event definitions.
 */

/**
 * System event encapsulation.
 */
class PFLIB_API TEvent {
public:
	/// Event codes.
	enum EEventCode {
		kIdle = 1,			///< Idle event processing time
		kNull,				///< EMPTY event
		kClose,				///< A close request (Alt-F4, clicking close button)
		kQuit,				///< A QUIT NOW event.
		kMouseDown,			///< Mouse down
		kExtendedMouseEvent,///< Right Mouse Button/Wheel down
		kMouseUp,			///< Mouse up
		kMouseMove,			///< Mouse move
		kMouseLeave,		///< Mouse has left the window. You must TWindowManager::SetCapture() the mouse to receive this message; note that TButton-derived classes automatically capture the mouse on mouse-over.
		kMouseHover,		///< Mouse has hovered over a point on the window.
		kKeyDown,			///< Key down
		kKeyUp,				///< Key up
		kChar,				///< translated character event
		kRedraw,			///< Redraw the screen now
		kTimer,				///< A timer has triggered
		kDisplayModeChange,	///< The display mode has changed
		kActivate,			///< Our application has activated/deactivated. mKey is set to 0 on deactivate, or 1 on activate.
		kFullScreenToggle	///< The user has toggled full screen mode. mKey is set to 0 on windowed mode, or non-zero on full screen mode.
	};

	/// Event key flags.
	enum EKeyFlags {
		kShift=1,		///< Shift is pressed.
		kControl=2,		///< Control (or command) is pressed.
		kAlt=4,			///< Alt key is pressed.
		kExtended=8,	///< Reserved.
	};

	/** @name Key Codes
	 *
     * these are ints so that they can be assigned
     * to platform specific contents without pulling platform
     * specific headers into client code
	 * @{
	 */
	static int32_t	kUp;
	static int32_t	kDown;
	static int32_t	kLeft;
	static int32_t	kRight;
	static int32_t	kEnter;
	static int32_t  kEscape;
	static int32_t  kTab;
	static int32_t  kPaste;
	static int32_t  kPageUp;
	static int32_t  kPageDown;
	static int32_t  kBackspace;
	static int32_t  kDelete;

	/*@}*/

	/**
	 * Type of event
	 *
	 * @see EEventCode
	 */
	int32_t			mType;

	/**
	 * Key for keyboard events.
	 */
	int32_t			mKey;

	/**
	 * Point for mouse events.
	 *
	 */
	TPoint		mPoint ;

	/**
	 * Flags for key events.
	 *
	 */
	EKeyFlags   mKeyFlags;
};

//------------------------------------------------------------------
#endif // EVENT_H_INCLUDED
