//------------------------------------------------------------------
// Copyright (c) 2004-2006 PlayFirst, Inc. All rights reserved.
//
// This material is the confidential trade secret and proprietary
// information of PlayFirst, Inc. It may not be reproduced, used, disclosed,
// sold or transferred without the prior written consent of PlayFirst, Inc.
// All rights reserved.
//------------------------------------------------------------------


/**
 * @file
 * Interface for class TChessPiece
 */


#ifndef CHESSPIECE_H_INCLUDED
#define CHESSPIECE_H_INCLUDED

// Playground headers can be included individually as below, or you can simply include
// pflib.h to get everything in one big header.
#ifndef WINDOW_H_INCLUDED
#include <pf/window.h>
#endif

#ifndef LIGHT_H_INCLUDED
#include <pf/light.h>
#endif

class TChessPiece : public TWindow
{
	PFTYPEDEF_DC(TChessPiece,TWindow)
public:
	/**
	 * Extended mouse button handler.
	 *
	 * @param point  Location of mouse event in client coordinates.
	 * @param event Event that happened.
	 *
	 * @return true if message was handled, false to keep searching
	 *         for a handler.
	 */
	virtual bool OnExtendedMouseEvent(const TPoint &point, TPlatform::ExtendedMouseEvents event);
	/**
	 * Mouse up handler.
	 *
	 * @param point      Location of mouse release in client coordinates.
	 *
	 * @return true if message was handled, false to keep searching
	 *         for a handler.
	 */
	virtual bool OnMouseUp(const TPoint &point);
	/**
	 * Mouse motion handler.
	 *
	 * @param point      Location of mouse in client coordinates.
	 *
	 * @return true if message was handled, false to keep searching
	 *         for a handler.
	 */
	virtual bool OnMouseMove(const TPoint &point);
	/**
	 * Notification that the mouse has left the window.
	 *
	 * @warning This message is only sent if SetCapture()
	 * has been called for this window previously.
	 *
	 * @return True if handled.
	 */
	virtual bool OnMouseLeave();
	/**
	 * Mouse down handler.
	 *
	 * @param point      Location of mouse press in client coordinates.
	 *
	 * @return true if message was handled, false to keep searching
	 *         for a handler.
	 */
	virtual bool OnMouseDown(const TPoint &point);
/// Default Constructor
	TChessPiece();

/// Destructor
	virtual ~TChessPiece();

	/**
	 * Called if you have initiated a window animation with
	 * TWindow::StartWindowAnimation.
	 *
	 * @return True to continue animating. False to stop.
	 */
	virtual bool OnTaskAnimate();

	/**
	 * Initialize the Window. Called by the system only in
	 * Lua initialization.
	 *
	 * When you create your own custom window, this is where you put
	 * your own custom initialization that needs to happen before
	 * children are created. Fundamental window initialization is
	 * handled in every class by this function, so <b>when you
	 * override this function you almost always want to call your
	 * base class to handle base class initialization.</b>
	 */
	virtual void Init(TWindowStyle &style);

	/**
	 * Draw the window. Derived classes will override this
	 * function and provide the draw functionality.
	 */
	virtual void Draw();
private:
	/// The chess piece model
	TModelRef mModel ;

	/// The material we'll use
	TMaterial mMat;

	/// The main light we'll use
	TLight mLight;

	/// A second, orbiting, light to use
	TLight mSpinLight;

	/// The texture for the chess piece.
	TTextureRef mTexture ;

	/// Our current rotation
	float mYaw;

	/// How long it's been since our last update.
	unsigned long mLastUpdate;

	bool mDragging ;
	TPoint mDragOrigin ;
};

#endif // CHESSPIECE_H_INCLUDED
