//------------------------------------------------------------------
// Copyright (c) 2004-2006 PlayFirst, Inc. All rights reserved.
//
// This material is the confidential trade secret and proprietary
// information of PlayFirst, Inc. It may not be reproduced, used, disclosed,
// sold or transferred without the prior written consent of PlayFirst, Inc.
// All rights reserved.
//------------------------------------------------------------------


#include "game.h"
#include "globaldefines.h"
#include "settings.h"

#include <pf/pflib.h>
#include <pf/soundinstance.h>

PFTYPEIMPL_DC(TGame);

oesSoundCallback::oesSoundCallback(TSoundInstanceRef _soundInstance, str _loop) {
	loop = _loop;
	soundInstance = _soundInstance;
}
void oesSoundCallback::OnComplete(TSoundInstanceRef _soundInstance, TSoundRef nextSound)
{

	nextSound = TSound::Get(loop);
	
	if(nextSound)
		soundInstance->SetCompleteAction(this,nextSound);

}

oesSoundtrack::oesSoundtrack(str _intro, str _loop) {
	intro = TSound::Get(_intro);
	loop = TSound::Get(_loop);

	soundInstance = intro->Play(0);
	
	soundCallback  = new oesSoundCallback(soundInstance,_loop);
	
	soundInstance->SetCompleteAction( soundCallback, loop );
}

void oesSoundtrack::setVolume(float volume) {
	soundInstance->SetVolume(volume);
}

oesSoundtrack::~oesSoundtrack() {
	delete soundCallback ;
}

TGame::TGame()
{
	timeCounter = TPlatform::GetInstance()->Timer();
	frameCounter = 0;
	delta = 0;
	previousTime = 0;
	zMove=0; xMove=0;
	leftPressed=0; rightPressed=0;

	mMyImage = TTexture::Get("backgrounds/mainmenu");
	//gd.load("leveldata/backscroll.oes");
	
	TPlatform::GetInstance()->SetConfig(TPlatform::kVsyncWindowedMode,"1");
	DEBUG_WRITE(("ENANNNAN: %d", TPlatform::GetInstance()->IsEnabled(TPlatform::kVsyncWindowedMode)));
	DEBUG_WRITE(("FIRST PEAK: %d", TPlatform::GetInstance()->IsEnabled(TPlatform::kFirstPeek)));
	DEBUG_WRITE(("FIRST PEAK: %c", TPlatform::GetInstance()->GetConfig("kFirstPeek") ));
	
	soundP = new oesSoundtrack("audio/Village/VillageIntro.ogg","audio/Village/VillageLoop.ogg");
	soundP2 = new oesSoundtrack("audio/Village/OutdoorIntro.ogg","audio/Village/OutdoorLoop.ogg");
	soundP->setVolume(0.7);

}

TGame::~TGame()
{
	delete soundP;
	delete soundP2;
}

void TGame::Draw()
{
	

	
	TRenderer * r = TRenderer::GetInstance();
	//r->SetTexture(mMyImage);
	
	TBegin2d begin2d;
		mMyImage->DrawSprite(0-xMove,380);
	mMyImage->DrawSprite(800-xMove,380);
	mMyImage->DrawSprite(1600-xMove,380);
	mMyImage->DrawSprite(2400-xMove,380);
	mMyImage->DrawSprite(3200-xMove,380);

	TMat4 viewMatrix;
	r->GetViewMatrix( &viewMatrix );
	//	viewMatrix.Scale(screenHeight/600.0, screenHeight/600.0,1);
	r->SetViewMatrix( &viewMatrix);
	
	int timePassed = (TPlatform::GetInstance()->Timer() - previousTime);
	if(timePassed>1) {
		previousTime = TPlatform::GetInstance()->Timer();
		delta = (float) timePassed / 10.0f;

		//if (on ==1) zMove+=(5*delta);
		//if (off ==1) zMove-=(5*delta);
		if (leftPressed) { xMove-=10; }//gd.camX -=(4*delta); }
		if (rightPressed) { xMove+=10; }//gd.camX+=(4*delta); }
		if (timePassed>30) {
			//DEBUG_WRITE(("OVER 30"));
		}
	}
	//gd.updateSpritePositions();
	frameCounter++;
	if(TPlatform::GetInstance()->Timer() - timeCounter > 10000) {
		timeCounter = TPlatform::GetInstance()->Timer();
		DEBUG_WRITE(("MY FPS %d", frameCounter/10));
		frameCounter=0;
	}
	begin2d.Done();
}

/**
 * Mouse motion handler.
 *
 * @param point      Location of mouse in client coordinates.
 *
 * @return true if message was handled, false to keep searching
 *         for a handler.
 */
bool TGame::OnMouseMove(const TPoint & /*point*/)
{
	return true ;
}

/**
 * Called if you have initiated a window animation with
 * TWindow::StartWindowAnimation.
 *
 * @return True to continue animating. False to stop.
 */
bool TGame::OnTaskAnimate()
{
    // And don't forget to update the screen
	TWindowManager::GetInstance()->InvalidateScreen();

    // Keep animating
	return true;
}

/**
 * Initialize the Window. Called by the system only in
 * Lua initialization.
 *
 * When you create your own custom window, this is where you put
 * your own custom initialization that needs to happen before
 * children are created. Fundamental window initialization is
 * handled in every class by this function, so <b>when you
 * override this function you almost always want to call your
 * base class to handle base class initialization.</b>
 */
void TGame::Init(TWindowStyle &style)
{
	TScript * s = TWindowManager::GetInstance()->GetScript();
	ScriptRegisterMemberDirect(s,"SendGameMessage",this,TGame::SendGameMessage);

    // Start the window animation
    StartWindowAnimation( 16 );

	mRows = style.GetInt("rows");
	mColumns= style.GetInt("columns");

	// Send keyboard events our way if no one else uses them
	FindParentModal()->SetDefaultFocus(this);
}

/**
 * Mouse down handler.
 *
 * @param point      Location of mouse press in client coordinates.
 *
 * @return true if message was handled, false to keep searching
 *         for a handler.
 */
bool TGame::OnMouseDown(const TPoint & /*point*/)
{

    return true;
}


void TGame::SendGameMessage( str message )
{
	if (message=="quit")
	{
		DEBUG_WRITE(("QUIT NOW"));
		// This handler could alternately queue up an "Are you sure you want to quit?" dialog
		// and that could, as its result, send a "reallyquit" message that actually quits the game.

		// Log a score for testing hiscores; nominally this code would live where you actually earned
		// the score rather than just every time you quit the game.
		TSettings::GetInstance()->GetHiscores()->LogScore(rand() % 10000, TSettings::GetInstance()->GetCurrentGameMode() == TSettings::eGameMode1, NULL);

		// Call the helper function in TSettings that pops all of the windows off the stack.
		// It assumes that you still have the forever loop in mainloop.lua that recreates the
		// main menu screen when it's been popped.
		TSettings::GetInstance()->QuitToMainMenu();
		TPlatform::GetInstance()->Exit(0)  ;
	}
}

/**
 * Raw key hit on keyboard.
 *
 * @param key    Key pressed on keyboard.
 * @param flags  TEvent::EKeyFlags mask representing the state of other keys on the keyboard
 *               when this key was hit.
 *
 * @return true if message was handled, false to keep searching
 *         for a handler.
 */
bool TGame::OnKeyDown(char key, uint32_t /*flags*/)
{
	if (key == TEvent::kEscape)
	{
		//TWindowManager::GetInstance()->DoLuaString(this,"DoModal('scripts/pause.lua')");
		SendGameMessage("quit");
		return true;
	}
	
	if (key == TEvent::kDown) {
		//on=1;
	} else if (key == TEvent::kUp) {
		//off = 1;
	} else if (key == TEvent::kLeft) {
		leftPressed=1;
	} else if (key == TEvent::kRight) {
		rightPressed=1;
	}

	return false;
}

bool TGame::OnKeyUp(char key)
{
	
	if (key == TEvent::kDown) {
		//on=0;
	} else if (key == TEvent::kUp) {
		//off=0;
	} else if (key == TEvent::kLeft) {
		leftPressed=0;
	} else if (key == TEvent::kRight) {
		rightPressed=0;
	}

	return false;
}