//------------------------------------------------------------------
// Copyright (c) 2004-2006 PlayFirst, Inc. All rights reserved.
//
// This material is the confidential trade secret and proprietary
// information of PlayFirst, Inc. It may not be reproduced, used, disclosed,
// sold or transferred without the prior written consent of PlayFirst, Inc. 
// All rights reserved.
//------------------------------------------------------------------

#ifndef GAME_H_INCLUDED
#define GAME_H_INCLUDED

#include <pf/window.h>
#include "readgamedata.h"
#include <pf/sound.h> 


class oesSoundCallback : public TSoundCallBack
{
public:
	oesSoundCallback(TSoundInstanceRef _soundInstance, str _loop);
	virtual void OnComplete(TSoundInstanceRef _soundInstance, TSoundRef nextSound );

private:
	str loop;
	TSoundInstanceRef soundInstance;
};

class oesSoundtrack {
public:
	TSoundRef intro;
	TSoundRef loop;
	TSoundInstanceRef soundInstance;
	oesSoundCallback  *soundCallback ;

	oesSoundtrack(str _intro, str _loop);
	void setVolume(float volume);
	~oesSoundtrack();

};

/**
 * Our example mini-game
 */
class TGame : public TWindow
{
	PFTYPEDEF_DC(TGame,TWindow)
public:
	TGame();
	~TGame();

	void Draw();

	/**
	 * Send a message to the game.
	 * 
	 * @param message Game message.
	 */
	void SendGameMessage( str message );

	/**
	 * Mouse down handler.
	 *
	 * @param point      Location of mouse press in client coordinates.
	 *
	 * @return true if message was handled, false to keep searching
	 *         for a handler.
	 */
	virtual bool OnMouseDown(const TPoint &point);
	/**
	 * Initialize the Window. Called by the system only in
	 * Lua initialization.
	 *
	 * When you create your own custom window, this is where you put
	 * your own custom initialization that needs to happen before
	 * children are created. Fundamental window initialization is
	 * handled in every class by this function, so <b>when you
	 * override this function you almost always want to call your
	 * base class to handle base class initialization.</b>
	 */
	virtual void Init(TWindowStyle &style);

	/**
	 * Called if you have initiated a window animation with
	 * TWindow::StartWindowAnimation.
	 *
	 * @return True to continue animating. False to stop.
	 */
	virtual bool OnTaskAnimate();

	/**
	 * Mouse motion handler.
	 *
	 * @param point      Location of mouse in client coordinates.
	 *
	 * @return true if message was handled, false to keep searching
	 *         for a handler.
	 */
	virtual bool OnMouseMove(const TPoint &point);

	/**
	 * Raw key hit on keyboard.
	 *
	 * @param key    Key pressed on keyboard.
	 * @param flags  TEvent::EKeyFlags mask representing the state of other keys on the keyboard
	 *               when this key was hit.
	 *
	 * @return true if message was handled, false to keep searching
	 *         for a handler.
	 */
	virtual bool OnKeyDown(char key, uint32_t flags);
	virtual bool OnKeyUp(char key);
private:
	
	int	mRows;		// Number of rows in the square
	int mColumns;	// Number of columns in the square

	TTextureRef mMyImage;
	TSoundRef mSoundRef;
	TSoundRef mSoundRef2;
	TSoundInstanceRef intro;
//	TSoundTesterCallBack soundT;
	
	//oesRender gd;
	bool leftPressed, rightPressed;
	TReal zMove, xMove;
	float delta;
	int previousTime;
	int timeCounter;
	int frameCounter;
	oesSoundtrack *soundP;
	oesSoundtrack *soundP2;

};

#endif