#define PLAYFIRST_MAIN
#define PLAYGROUND_INIT

#include "main.h"
#include "key.h"

#include <pf/windowmanager.h>
#include <pf/image.h>
#include <pf/button.h>
#include <pf/texture.h>
#include <pf/event.h>
#include <pf/task.h>
#include <pf/stringtable.h>
#include <pf/screen.h>
#include <pf/slider.h>
#include <pf/script.h>

int screenWidth;
int screenHeight;
oesGame *game;

bool oesAnimTask::Animate() {
	game->drawScreen();
	return 1;
}


// Early initialization of Playground values. TPlatform has not yet
// been initialized here, so only static functions on TPlatform are
// legal! At present this is basically only SetConfig(). This function
// is really designed only for setting up values that TPlatform will need
// during initialization; most application initialization still belongs in
// Main().
void PlaygroundInit()
{
	// Set the name of the game. This name will be used in the user: and
	// common: data folder names, as well as the handle given to the high
	// score server. To just change the high score server handle, set this
	// value in Main below instead (or in addition to give it a different
	// name).
	TPlatform::SetConfig( TPlatform::kGameName, "gameEngine" );

	// Set the PlayFirst game information. All PlayFirst games MUST set the
	// information below in order to successfully communicate with PlayFirst
	// services, such as the hiscore system. Removing this code will cause
	// the hiscore system to not work.
	// DO NOT REMOVE THIS CODE IF YOU PLAN ON USING PLAYFIRST SERVICES
	// ---------------------------------------------------------------
	TPlatform::SetConfig(TPlatform::kEncryptionKey,ENCRYPTION_KEY);
	TPlatform::SetConfig(TPlatform::kPFGameHandle,PFGAMEHANDLE);
	for (int i = 0; i < sizeof(PFGAMEMODENAMES)/sizeof(char *); i++)
	{
		str name = str::getFormatted("%s%d", TPlatform::kPFGameModeName, i+1);
		TPlatform::SetConfig(name,PFGAMEMODENAMES[i]);
	}

	for (int i = 0; i < sizeof(PFGAMEMEDALNAMES)/sizeof(char *); i++)
	{
		str name = str::getFormatted("%s%d", TPlatform::kPFGameMedalName, i+1);
		TPlatform::SetConfig(name,PFGAMEMEDALNAMES[i]);
	}
	// ---------------------------------------------------------------
}


oesGame::oesGame() {

	TPlatform *platform = TPlatform::GetInstance();

	quit=0;
	xStretchRatio = yStretchRatio = 1;
	gameActive=1;
	mLastUpdate=0;

	fullScreen = (platform->GetConfig( "fullScreen",  "0" ).to_int()!=0);
	stretchScreen = (platform->GetConfig( "stretchScreen",  "0" ).to_int()!=0);

	// If fullscreen, load width height from prefs, 800x500 in windowed mode
	if( fullScreen ) {
		screenWidth = platform->GetConfig( "screenWidth",  "800" ).to_int();
		screenHeight = platform->GetConfig( "screenHeight",  "600" ).to_int();
		if(screenWidth!=800) {
			xStretchRatio = screenWidth / float(800);
			yStretchRatio = screenHeight / float(600);
		}
	} else {
		screenWidth=800;
		screenHeight=500;
	}

	platform->SetDisplay(screenWidth, screenHeight, fullScreen);
	//platform->SetDisplay(800,600,1);

	if( platform->GetConfig( "vSync",  "0" ).to_int() ) {
		platform->SetConfig(TPlatform::kVsyncWindowedMode,"1");
	}
	//platform->SetConfig(TPlatform::kVsyncWindowedMode,"1");
	//TRenderer::GetInstance()->SetOption("fps", "0");
	
	leftPressed = 0; rightPressed = 0;
	//previousTime = TPlatform::GetInstance()->Timer();

	// Default to the new subtractive renderer behavior, which is
	// correctly cross-platform to the Mac.
	TRenderer::GetInstance()->SetOption("new_subtractive","1");

	// Set the application name
	platform->SetWindowTitle(
		platform->GetStringTable()->GetString("windowtitle").c_str()
		);

	platform->SetCursor( TTexture::GetSimple("cursor/cursor"), TPoint(1,1) );

	platform->SetCursor( TTexture::Get("cursor/thumb.png"), TPoint(12,2), true );

	const char *level = platform->GetConfig( "level",  "leveldata/village.oes").c_str();
	gameRenderer.load( level );
	

	anim = new oesAnimTask();
	anim->SetDelay(0);
	TPlatform::GetInstance()->AdoptTask(anim);
	
	TPlatform::GetInstance()->GetWindowManager()->GetScreen()->GetClock()->Reset();
	TPlatform::GetInstance()->GetWindowManager()->GetScreen()->GetClock()->Start();
	timeP=0; numFrames=0;
}

oesGame::~oesGame() {
	//delete anim;
}



void oesGame::drawScreen() {
	
	TRenderer * r = TRenderer::GetInstance();

	if(leftPressed) { if(gameRenderer.camX>0) gameRenderer.camX -=(4); }
	
	if(rightPressed) { if(gameRenderer.camX<4000) gameRenderer.camX +=(4); }

	if(gameActive) { if(r->BeginDraw  ( 0 )) {
			
		r->Begin2d();

		// Set up view matrix to scale game screen to any resolution
		// Add black borders if stretchScreen disabled

		TMat4 viewMatrix, origViewMatrix;
		r->GetViewMatrix( &origViewMatrix );
		viewMatrix = origViewMatrix;

		if(fullScreen) {
			if (stretchScreen) {
				viewMatrix.Translate( TVec3(0, -100, 0) );
				viewMatrix.Scale(1, 1.2f, 1);
			} else {
				viewMatrix.Translate( TVec3(0, -50, 0) );
			}
			viewMatrix.Scale(xStretchRatio, yStretchRatio, 1);
		} else {
			viewMatrix.Translate( TVec3(0, -100, 0) );

		}

		r->SetViewMatrix( &viewMatrix);
		
		// Update and render all objects
		gameRenderer.updateSpritePositions();
		
		viewMatrix = origViewMatrix;
		viewMatrix.Scale(xStretchRatio, yStretchRatio, 1);
		r->SetViewMatrix( &viewMatrix);

		// Draw borders
		if (fullScreen && (!stretchScreen)) {
			r->FillRect( TURect(0,0,800,50), TColor(0,0,0,1) );
			r->FillRect( TURect(0,550,800,600), TColor(0,0,0,1) );
		}
		
		r->End2d();	
		r->EndDraw(1);
	
	} }
}

void oesGame::handleEvents() {
	
	TEvent event;

	TPlatform::GetInstance()->GetEvent(&event);
	
	if(event.mType == TEvent::kKeyDown) { 
		if(event.mKey == TEvent::kLeft) {
			leftPressed=1;
		}
		if(event.mKey == TEvent::kRight) {
			rightPressed=1;
		}
		if(event.mKey == TEvent::kEscape) {
			quit=1;
		}

	}

	if(event.mType == TEvent::kKeyUp) { 
		if(event.mKey == TEvent::kLeft) {
			leftPressed=0;
		}
		if(event.mKey == TEvent::kRight) {
			rightPressed=0;
		}
	}


	if(event.mType == TEvent::kQuit)
	{
		quit=1;
	}

	if (event.mType == TEvent::kClose)
	{
		quit=1;
	}

	// Set a delay before making the game active and drawing,
	// to avoid error of invalid DX surface when minimizing the game
	// possibly due to DX needing some time to recreate/reload surfaces
	if(!TPlatform::GetInstance()->IsForeground()) { gameActive=0; gameActiveCounter=0; }
	else {
		if(gameActive==0) {
			gameActiveCounter++;
			if(gameActiveCounter==100) {
				gameActive=1;
			}
		}
	}
}




//---------------------------

/// Main application entry point
/// Uncomment cmdLine below if you need to process the command line
void Main(TPlatform* pPlatform, const char* /*cmdLine*/ )
{
	game = new oesGame();

	while( game->getQuitStatus() ) {

		game->handleEvents();

	}	

	delete game;
	pPlatform->Exit();
}

