#include "readgamedata.h"
#include <pf/debug.h>
#include <pf/str.h>

oesRender::oesRender()
{}

oesRender::~oesRender()
{
	delete dSpriteProperties;
	delete[] textureList;
	delete spriteDrawList;
	levelAssets1.Release();
}

void oesRender::readHeader(const char *level)
{
	int timeGone = TPlatform::GetInstance()->Timer();
	
	// Open asci file
	fileHandle = pf_open( level, kReadText );

	VERIFY(( fileHandle != NULL ));

	// Buffer for storing header data
	char headerBuffer[buffersize];

	// Get the header data
	VERIFY(( pf_gets( fileHandle, headerBuffer, buffersize ) != NULL ));
	sscanf ( headerBuffer, "%d %d %s", &numFiles, &numObjects, filePath);

	spriteDrawList = new int[numObjects];
	textureList = new TTextureRef[numFiles];

	// Get level texture list
	char texture[textureBufferSize];
	unsigned int height, width;
	for(int i=0; i<numFiles; i++) {
		VERIFY (( pf_gets( fileHandle, texture, textureBufferSize ) != NULL ));
		sscanf( texture, "%s", texture);

		// Get full path
		str finalPath(filePath);
		finalPath.append(texture,textureBufferSize);
		levelAssets1.AddAsset(finalPath);
		textureList[i] = levelAssets1.GetTexture(finalPath);
		height = levelAssets1.GetTexture(finalPath)->GetInternalSize().x;
		width = levelAssets1.GetTexture(finalPath)->GetInternalSize().y;
	}
		
	DEBUG_WRITE(( "Time to load textures: %d", TPlatform::GetInstance()->Timer() - timeGone ));	
}

void oesRender::load(const char *level)
{

	readHeader(level);
	int timeGone = TPlatform::GetInstance()->Timer();

	dSpriteProperties = new spriteProperties[numObjects];
		
	camX = 0;
	camY = 0;

	int i;
	int texture;

	// Create char list as long as num objects
	buffer = new char* [numObjects];

	// Allocate buffer for each object
	
	for(i=0; i<numObjects; i++) {
		buffer[i] = new char[buffersize];
	}
	
	// Read values for each object
	for (i=0; i<numObjects; ) {
		VERIFY (( pf_gets( fileHandle, buffer[i], buffersize ) != NULL ));
	
		sscanf ( buffer[i],"%f %f %f %f %f %f %f %f %d",
			&x, &y, &z, &scale, &r, &g, &b, &a, &texture);
		
		spriteDrawList[i] = texture;

		y = (y-600)*-1;

		dSpriteProperties[i].x = x;
		dSpriteProperties[i].y = y;

		dSpriteProperties[i].r = r;
		dSpriteProperties[i].g = g;
		dSpriteProperties[i].b = b;
		dSpriteProperties[i].a = a;
		dSpriteProperties[i].z = z;
		dSpriteProperties[i].scale = scale;
		dSpriteProperties[i].texture = texture;
		dSpriteProperties[i].hWidth = (textureList[texture]->GetWidth()/2);
		dSpriteProperties[i].hHeight = (textureList[texture]->GetHeight()/2);

		i++;
	}

	// Delete buffer and close file
	for(i=0; i<numObjects; i++) {
		delete buffer[i];
	}
	delete buffer;
	pf_close( fileHandle );

	DEBUG_WRITE(( "Time to read level object data: %d", TPlatform::GetInstance()->Timer() - timeGone ));
}


void oesRender::updateSpritePositions() {
	float x, y, distanceScale, scale;
	float camXcenter, camYcenter;
	int numDrawn;
	std::vector<spriteProperties*> spriteDrawList;
	numDrawn=0;

	// loop through all objects, find their position
	for(int i=0; i<numObjects; i++) {

		distanceScale = 800 / (800- dSpriteProperties[i].z);
		scale = distanceScale * dSpriteProperties[i].scale;

		camXcenter = camX + 400;
		camYcenter = camY + 300;

		x = ( (dSpriteProperties[i].x - camXcenter)*distanceScale ) + camXcenter;
		y = ( (dSpriteProperties[i].y - camYcenter)*distanceScale ) + camYcenter;
		
		dSpriteProperties[i].xT = x-camX;
		dSpriteProperties[i].yT = y-camY;
		dSpriteProperties[i].scaleT = scale;

		// add to sprite list (objects to draw) if within the screen
		if ( (x-(dSpriteProperties[i].hWidth*scale)<800+camX) && ((x+dSpriteProperties[i].hWidth*scale) > 0+camX) ) {
			//if(dSpriteProperties[i].z >-10)
			spriteDrawList.push_back(&dSpriteProperties[i]);
		} else {
		}
			
	}

	// sort the sprite list according to Z
	std::sort(spriteDrawList.begin(), spriteDrawList.end(), sortZ );	

	// draw the sprite list
	for (unsigned int i = 0; i < spriteDrawList.size(); i++) {
		
		TDrawSpec drawSpec( TVec2( spriteDrawList[i]->xT, spriteDrawList[i]->yT ), spriteDrawList[i]->a , spriteDrawList[i]->scaleT );
		drawSpec.mTint = TColor(spriteDrawList[i]->r, spriteDrawList[i]->g, spriteDrawList[i]->b, spriteDrawList[i]->a);

		int textureIndex = spriteDrawList[i]->texture;
			textureList[ textureIndex ]->DrawSprite(drawSpec);
			numDrawn++;
	}
}

bool sortZ(const spriteProperties* lhs, const spriteProperties* rhs)
{
	if (lhs->z == rhs->z) return lhs->x < rhs->x;
	return lhs->z < rhs->z;
}

gameAssets::gameAssets()
{}

gameAssets::~gameAssets()
{}