//------------------------------------------------------------------
// Copyright (c) 2004-2006 PlayFirst, Inc. All rights reserved.
//
// This material is the confidential trade secret and proprietary
// information of PlayFirst, Inc. It may not be reproduced, used, disclosed,
// sold or transferred without the prior written consent of PlayFirst, Inc.
// All rights reserved.
//------------------------------------------------------------------

#include "settings.h"
#include "globaldefines.h"

#include <pf/pflib.h>
#include <pf/windowmanager.h>
#include <pf/script.h>
#include <pf/soundmanager.h>
#include <pf/stringtable.h>

#include "key.h"



TSettings *gSettings = NULL;

TSettings *TSettings::GetInstance()
{
	return gSettings;
}

TSettings *TSettings::CreateSettings()
{
	if (gSettings)
	{
		delete gSettings;
	}
	gSettings = new TSettings();
	return gSettings;
}
void TSettings::DeleteSettings()
{
	if (gSettings)
	{
		delete gSettings;
	}
	gSettings = NULL;
}

TSettings::TSettings()
{
	// Register some useful global GUI script functions. These functions will be available
	// in the global Lua GUI script.
	TScript * s = TWindowManager::GetInstance()->GetScript();
	ScriptRegisterMemberDirect( s, "ToggleFullScreen", this, TSettings::ToggleFullScreen );
	ScriptRegisterMemberDirect( s, "ToggleSoundMute", this, TSettings::ToggleSoundMute );
	ScriptRegisterMemberDirect( s, "SetCurrentGameMode", this, TSettings::SetCurrentGameMode );
	ScriptRegisterMemberDirect( s, "GetNumUsers", this, TSettings::GetNumUsers );
	ScriptRegisterMemberDirect( s, "CreateNewUser", this, TSettings::CreateNewUser );
	ScriptRegisterMemberDirect( s, "SetCurrentUser", this, TSettings::SetCurrentUser );
	ScriptRegisterMemberDirect( s, "ChangeCurrentUserName", this, TSettings::ChangeCurrentUserName );
	ScriptRegisterMemberDirect( s, "DeleteUser", this, TSettings::DeleteUser );
	ScriptRegisterMemberDirect( s, "IsNameInUse", this, TSettings::IsNameInUse );
	ScriptRegisterMemberDirect( s, "QuitToMainMenu", this, TSettings::QuitToMainMenu );

	

	// Load in saved preferences file
	mPreferences = new TPrefs();
	mHiscores = new TPfHiscores();

	SetCurrentGameMode(0);
	SetCurrentUser(GetCurrentUser());

}

TSettings::~TSettings()
{
	delete mPreferences;
	delete mHiscores;
}

#if _DEBUG
#define DEFAULT_FULLSCREEN 0
#else
#define DEFAULT_FULLSCREEN 1
#endif

void TSettings::InitGameToSettings()
{
	// Set up the application display
	bool fullScreen = 0;

	if (mPreferences->GetInt("fullscreen", DEFAULT_FULLSCREEN) == 1)
	{
		fullScreen = true;
	}

	//TPlatform::GetInstance()->SetDisplay(screenWidth, screenHeight, fullScreen);

	SetSoundForCurrentUser();
}

bool TSettings::IsFullScreen()
{
	return TPlatform::GetInstance()->IsFullscreen();
}

void TSettings::UpdateFullScreen()
{
	bool full = TPlatform::GetInstance()->IsFullscreen();

	if (!full)
	{
		mPreferences->SetInt("fullscreen", 0);
	}
	else
	{
		mPreferences->SetInt("fullscreen", 1);
	}
}

bool TSettings::ToggleFullScreen()
{
	bool full = TPlatform::GetInstance()->IsFullscreen();

	bool success = true;

	if (!full)
	{
		if (TPlatform::GetInstance()->SetFullscreen(true))
		{
			mPreferences->SetInt("fullscreen", 1);
		}
		else
		{
			success = false;
		}
	}
	else
	{
		if (TPlatform::GetInstance()->SetFullscreen(false))
		{
			mPreferences->SetInt("fullscreen", 0);
		}
		else
		{
			success = false;
		}
	}

	return success;
}


void TSettings::ToggleSoundMute()
{

	int currentUser = mPreferences->GetInt("curuser", TPrefs::kGlobalIndex, TPrefs::kGlobalIndex);
	int soundOff = mPreferences->GetInt("soundoff", 0, currentUser);
	soundOff = (soundOff+1)%2;
	mPreferences->SetInt("soundoff", soundOff, currentUser);

	if(soundOff == 0)
	{
		TPlatform::GetInstance()->GetSoundManager()->SetVolume(1);
	}
	else
	{
		TPlatform::GetInstance()->GetSoundManager()->SetVolume(0);
	}
}

void TSettings::SetCurrentGameMode(int gameMode)
{
	if (gameMode < 0 || gameMode >= eMaxGameModes) return;

	mGameMode = (eGameMode)gameMode;

	mHiscores->SetProperty(TPfHiscores::eGameMode, PFGAMEMODENAMES[mGameMode]);
}


void TSettings::CreateNewUser(str userName)
{
	int numUsers = GetNumUsers();
	mPreferences->SetStr("name", userName, numUsers);
	// copy the sound settings from the current user so that the sound isn't jarring
	// when creating a new user
	int currentUser = GetCurrentUser();
	int soundVal = mPreferences->GetInt("soundlevel", DEFAULT_SOUND_VOLUME, currentUser);
	int musicVal = mPreferences->GetInt("musiclevel", DEFAULT_SOUND_VOLUME, currentUser);
	int soundOff = mPreferences->GetInt("soundoff", 0, currentUser);
	mPreferences->SetInt("soundlevel", soundVal, numUsers);
	mPreferences->SetInt("musiclevel", musicVal, numUsers);
	mPreferences->SetInt("soundoff", soundOff, numUsers);

	SetCurrentUser(numUsers);

}

void TSettings::ChangeCurrentUserName(str newName)
{
	int currentUser = mPreferences->GetInt("curuser", 0, TPrefs::kGlobalIndex);
	mPreferences->SetStr("name", newName, currentUser);
}

str TSettings::GetCurrentUserName()
{
	int currentUser = mPreferences->GetInt("curuser", 0, TPrefs::kGlobalIndex);
	str username = mPreferences->GetStr("name", "", currentUser);
	return username;
}

str TSettings::GetUserName(int index)
{
	return mPreferences->GetStr("name", "", index);
}

void TSettings::SetCurrentUser(int userNum)
{
	int numUsers = GetNumUsers();
	if (userNum >= numUsers) return;

	mPreferences->SetInt("curuser", userNum, TPrefs::kGlobalIndex);
	if (userNum >= 0)
	{
		mHiscores->SetProperty(TPfHiscores::ePlayerName, GetCurrentUserName().c_str());
		SetSoundForCurrentUser();
	}

}

int TSettings::GetCurrentUser()
{
	return mPreferences->GetInt("curuser", TPrefs::kGlobalIndex, TPrefs::kGlobalIndex);
}

int TSettings::GetNumUsers()
{
	return mPreferences->GetNumUsers();
}

void TSettings::SetSoundForCurrentUser()
{

	// initialize volume settings
	int currentUser = mPreferences->GetInt("curuser", TPrefs::kGlobalIndex, TPrefs::kGlobalIndex);
	int soundVal = mPreferences->GetInt("soundlevel", DEFAULT_SOUND_VOLUME, currentUser);
	int musicVal = mPreferences->GetInt("musiclevel", DEFAULT_SOUND_VOLUME, currentUser);

	TPlatform::GetInstance()->GetSoundManager()->SetTypeVolume(kSFXSoundGroup, (float)soundVal/1000.0f);
	TPlatform::GetInstance()->GetSoundManager()->SetTypeVolume(TSound::kButtonSound, (float)soundVal/1000.0f);
	TPlatform::GetInstance()->GetSoundManager()->SetTypeVolume(kMusicSoundGroup, (float)musicVal/1000.0f);


	int soundOff = mPreferences->GetInt("soundoff", 0, currentUser);
	if(soundOff == 0)
	{
		TPlatform::GetInstance()->GetSoundManager()->SetVolume(1);
	}
	else
	{
		TPlatform::GetInstance()->GetSoundManager()->SetVolume(0);
	}
}

void TSettings::DeleteUser(int userNum)
{
	mPreferences->DeleteUser(userNum);
	userNum--;
	if (userNum < 0)
	{
		userNum = GetNumUsers()-1;
	}
	SetCurrentUser(userNum);
}

bool TSettings::IsNameInUse(str name)
{
	int numUsers = GetNumUsers();

	for (int i = 0; i < numUsers; i++)
	{
		if (name == GetUserName(i))
		{
			return true;
		}
	}

	return false;
}

void TSettings::QuitToMainMenu()
{
	TWindowManager * wm = TWindowManager::GetInstance();

	while (wm->GetTopModalWindow() && wm->GetTopModalWindow()->GetParent())
	{
		wm->PopModal( wm->GetTopModalWindow()->GetID() );
	}
}
