//------------------------------------------------------------------
// Copyright (c) 2004-2006 PlayFirst, Inc. All rights reserved.
//
// This material is the confidential trade secret and proprietary
// information of PlayFirst, Inc. It may not be reproduced, used, disclosed,
// sold or transferred without the prior written consent of PlayFirst, Inc.
// All rights reserved.
//------------------------------------------------------------------


/**
 * @file
 * Implementation for class TSwarm
 */

#include "swarm.h"
#include <pf/pflib.h>
#include <pf/animatedtexture.h>
#include <pf/animatedsprite.h>

PFTYPEIMPL_DC(TSwarm);

TSwarm::TSwarm() :
	mLastUpdate(0)
{
	mSpriteHolder = TSprite::Create();

	mTexture = TAnimatedTexture::Get("anim/cardinal.xml");
	if (mTexture)
	{
		for (int i = 0 ;i<kNumSprites; ++i)
		{
			mSprites[i] = TAnimatedSprite::Create(i);
			mSprites[i]->SetTexture(mTexture);
			mSprites[i]->Play();
			TDrawSpec drawSpec(
				TVec2( (float)((float)kBoundary+i*(float)(kWidth-kBoundary*2)/10.0F),
					   (float)((float)kBoundary+i*(float)(kHeight-kBoundary*2)/10.0F) ),
				1,0.8F
				);
			drawSpec.mMatrix.Scale( 0.5F + TPlatform::GetInstance()->Rand()%1000/1000.0F );

			mSprites[i]->GetDrawSpec()= drawSpec;
			mVelocity[i]= TVec2(0,0);

			mSpriteHolder->AddChild(mSprites[i]);
		}
	}
	mHitCount = 0;
}

TSwarm::~TSwarm()
{

}

/**
 * Draw the window. Derived classes will override this
 * function and provide the draw functionality.
 */
void TSwarm::Draw()
{
	TRenderer * r = TRenderer::GetInstance();
	TBegin2d begin2d ;

	mSpriteHolder->Draw();

	mParticles.Draw( TVec3(0,0,0) );
	mParticles2.Draw( TVec3(0,0,0) );

    TRect screenRect ;
    GetClientRect(&screenRect);

    TColor color(1,0,0,1);
	r->FillRect(TURect(screenRect.mX1,screenRect.mY1,screenRect.mX1+2,screenRect.mY2),color);
	r->FillRect(TURect(screenRect.mX1,screenRect.mY1,screenRect.mX2,screenRect.mY1+2),color);
	r->FillRect(TURect(screenRect.mX2-2,screenRect.mY1,screenRect.mX2,screenRect.mY2),color);
	r->FillRect(TURect(screenRect.mX1,screenRect.mY2-2,screenRect.mX2,screenRect.mY2),color);
}

/**
 * Initialize the Window. Called by the system only in
 * Lua initialization.
 *
 * When you create your own custom window, this is where you put
 * your own custom initialization that needs to happen before
 * children are created. Fundamental window initialization is
 * handled in every class by this function, so <b>when you
 * override this function you almost always want to call your
 * base class to handle base class initialization.</b>
 */
void TSwarm::Init(TWindowStyle & style )
{
	StartWindowAnimation(16);

	// Send keyboard events our way if no one else uses them
	FindParentModal()->SetDefaultFocus(this);
	mParticles.RegisterDataSource("dLocus",&mSwarmLocus);
	mParticles2.RegisterDataSource("dLocus",&mSwarmLocus);
	mParticles.Init("gamepiece/spinshine.lua");
	mParticles2.Init("gamepiece/spinstars.lua");

	mParticles2.GetScript()->DoLuaString("gActive = false;");

	mEdgeBounce = (TReal)style.GetNumber("edgebounce");
	mDamping = (TReal)style.GetNumber("damping");
	mPull = (TReal)style.GetNumber("butterflypull");
	mPush = (TReal)style.GetNumber("butterflyrepel");

	// Bind our sprite animations to the window anim clock,
	// which in turn was initialized (by the library) from
	// the parent TModalWindow::GetClock().
	TClock * clock = GetWindowAnim()->GetClock();
	for (int i=0 ;i <kNumSprites; ++i)
	{
		mSprites[i]->SetClock( clock );
	}

}

/**
 * Mouse down handler.
 *
 * @param point      Location of mouse press in client coordinates.
 *
 * @return true if message was handled, false to keep searching
 *         for a handler.
 */
bool TSwarm::OnMouseDown(const TPoint &point)
{
	if (mHitCount==kNumSprites)
	{
		for (int i=0; i<kNumSprites; ++i)
		{
			mSprites[i]->SetVisible(true);
		}
		mHitCount=0;
	}
	else
	{
		for (int i=kNumSprites-1; i>=0; --i)
		{
			if (mSprites[i]->IsVisible())
			{
				if (mSprites[i]->HitTest(point,mSpriteHolder->GetDrawSpec(),1))
				{
					mSprites[i]->SetVisible(false);
					mHitCount++;
					break;
				}
			}
		}
	}
	return true;
}

/**
 * Mouse motion handler.
 *
 * @param point      Location of mouse in client coordinates.
 *
 * @return true if message was handled, false to keep searching
 *         for a handler.
 */
bool TSwarm::OnMouseMove(const TPoint &point)
{
	mLastMouse = TVec2((TReal)point.mX,(TReal)point.mY);
	mSwarmLocus.mPosition = mLastMouse ;
	return true;
}

/**
 * Called if you have initiated a window animation with
 * TWindow::StartWindowAnimation.
 *
 * @return True to continue animating. False to stop.
 */
bool TSwarm::OnTaskAnimate()
{
	TWindowManager::GetInstance()->InvalidateScreen();

	uint32_t now = GetWindowAnim()->GetClock()->GetTime();
	TReal elapsedTime = (float)(now - mLastUpdate)/1000.0f;
	mLastUpdate = now;

	for (int i=0; i<kNumSprites; ++i)
	{
		TVec3 delta = TVec3(mLastMouse)-mSprites[i]->GetDrawSpec().mMatrix[2] ;

		mVelocity[i] += TVec2(delta * elapsedTime * mPull);
		mSprites[i]->GetDrawSpec().mMatrix =
			TMat3::GetRotation(mVelocity[i].x *0.0001F + mVelocity[i].y *0.0002F)*
			mSprites[i]->GetDrawSpec().mMatrix ;

		if (i>0)
		{
			delta = mSprites[i-1]->GetDrawSpec().mMatrix[2]-mSprites[i]->GetDrawSpec().mMatrix[2] ;
			if (delta.x==0 && delta.y==0)
			{
				delta.x = 1 ;
			}
			float lengthSq = delta.x*delta.x + delta.y*delta.y ;
			if (lengthSq<1)
				lengthSq = 1;

			TVec3 push = delta * (1.0F/lengthSq) * mPush * 40 * elapsedTime;
			mSprites[i-1]->GetDrawSpec().mMatrix[2] += push ;
			mSprites[i]->GetDrawSpec().mMatrix[2] -= push ;
		}

		mSprites[i]->GetDrawSpec().mMatrix[2] += TVec3(mVelocity[i] * elapsedTime,0);
		mVelocity[i] -= mVelocity[i] * mDamping * elapsedTime;

		if (mSprites[i]->GetDrawSpec().mMatrix[2].x<kBoundary && mVelocity[i].x<0)
		{
			mSprites[i]->GetDrawSpec().mMatrix[2].x=kBoundary ;
			mVelocity[i].x= - mVelocity[i].x * mEdgeBounce;
		}
		if (mSprites[i]->GetDrawSpec().mMatrix[2].y<kBoundary  && mVelocity[i].y<0)
		{
			mSprites[i]->GetDrawSpec().mMatrix[2].y=kBoundary ;
			mVelocity[i].y= - mVelocity[i].y * mEdgeBounce;
		}
		if (mSprites[i]->GetDrawSpec().mMatrix[2].x>kWidth-kBoundary && mVelocity[i].x>0)
		{
			mSprites[i]->GetDrawSpec().mMatrix[2].x=(float)kWidth-kBoundary ;
			mVelocity[i].x= - mVelocity[i].x * mEdgeBounce;
		}
		if (mSprites[i]->GetDrawSpec().mMatrix[2].y>kHeight-kBoundary && mVelocity[i].y>0)
		{
			mSprites[i]->GetDrawSpec().mMatrix[2].y=(float)kHeight-kBoundary ;
			mVelocity[i].y= - mVelocity[i].y * mEdgeBounce;
		}
	}
	mParticles.Update(16);
	mParticles2.Update(16);
	return true;
}

/**
 * Raw key hit on keyboard.
 *
 * @param key    Key pressed on keyboard.
 * @param flags  TEvent::EKeyFlags mask representing the state of other keys on the keyboard
 *               when this key was hit.
 *
 * @return true if message was handled, false to keep searching
 *         for a handler.
 */
bool TSwarm::OnKeyDown(char key, uint32_t /*flags*/)
{
	if (key == TEvent::kEscape)
	{
		TWindowManager::GetInstance()->DoLuaString(this,"DoModal('scripts/pause.lua')");
		return true;
	}

	return false;
}

PFTYPEIMPL_DC(SwarmLocus);
/**
 * Initialize parameters.
 *
 * @param table  Incoming parameters.
 */
uint8_t SwarmLocus::GetReturnSize(int /*paramCount*/, uint8_t * /*sizes*/)
{
	return 2;
}

/**
 * Process function or fetch data
 *
 * @param particle Particle being processed.
 * @param params   [in/out] parameters
 *
 * @return True on success; false if parameters are
 *         incorrect.
 */
bool SwarmLocus::Process(TParticleState & /*particle*/, TParticleMachineState &params)
{
	params.Push( mPosition );
	return true;
}

/**
 * Extended mouse button handler.
 *
 * @param point  Location of mouse event in client coordinates.
 * @param event Event that happened.
 *
 * @return true if message was handled, false to keep searching
 *         for a handler.
 */
bool TSwarm::OnExtendedMouseEvent(const TPoint & /*point*/, TPlatform::ExtendedMouseEvents event)
{
	if (event==TPlatform::kMouseRightDown)
	{
		mParticles.GetScript()->DoLuaString("gActive = not gActive;");
		mParticles2.GetScript()->DoLuaString("gActive = not gActive;");

		return true;
	}

	return false;
}

/**
 * Handle any initialization or setup that is required when
 * this window is assigned to a new parent. No initialization
 * of the window has happened prior to this call.
 *
 * @return True on success; false on failure.
 *
 * @see Init
 * @see PostChildrenInit
 */
bool TSwarm::OnNewParent()
{
	mLastUpdate = FindParentModal()->GetClock()->GetTime();
	return true;
}
