//------------------------------------------------------------------
// Copyright (c) 2004-2006 PlayFirst, Inc. All rights reserved.
//
// This material is the confidential trade secret and proprietary
// information of PlayFirst, Inc. It may not be reproduced, used, disclosed,
// sold or transferred without the prior written consent of PlayFirst, Inc.
// All rights reserved.
//------------------------------------------------------------------


/**
 * @file
 * Interface for class TSwarm
 */


#ifndef SWARM_H_INCLUDED
#define SWARM_H_INCLUDED

#ifndef WINDOW_H_INCLUDED
#include <pf/window.h>
#endif

#ifndef LUAPARTICLESYSTEM_H_INCLUDED
#include <pf/luaparticlesystem.h>
#endif

class SwarmLocus : public TParticleFunction
{
	PFTYPEDEF_DC(SwarmLocus,TParticleFunction);
public:
	SwarmLocus() : mPosition(0,0)
	{

	}

	/**
	 * Process function or fetch data
	 *
	 * @param particle Particle being processed.
	 * @param params   [in/out] parameters
	 *
	 * @return True on success; false if parameters are
	 *         incorrect.
	 */
	virtual bool Process(TParticleState &particle, TParticleMachineState &params);

	/**
	 * Initialize parameters.
	 *
	 * @param table  Incoming parameters.
	 */
	virtual uint8_t GetReturnSize(int paramCount, uint8_t *sizes);

	TVec2 mPosition;
};

class TSwarm : public TWindow
{
	PFTYPEDEF_DC(TSwarm,TWindow)

public:
	/**
	 * Handle any initialization or setup that is required when
	 * this window is assigned to a new parent. No initialization
	 * of the window has happened prior to this call.
	 *
	 * @return True on success; false on failure.
	 *
	 * @see Init
	 * @see PostChildrenInit
	 */
	virtual bool OnNewParent();
	/**
	 * Extended mouse button handler.
	 *
	 * @param point  Location of mouse event in client coordinates.
	 * @param event Event that happened.
	 *
	 * @return true if message was handled, false to keep searching
	 *         for a handler.
	 */
	virtual bool OnExtendedMouseEvent(const TPoint &point, TPlatform::ExtendedMouseEvents event);
/// Default Constructor
	TSwarm();

/// Destructor
	virtual ~TSwarm();

	/**
	 * Called if you have initiated a window animation with
	 * TWindow::StartWindowAnimation.
	 *
	 * @return True to continue animating. False to stop.
	 */
	virtual bool OnTaskAnimate();
	/**
	 * Mouse motion handler.
	 *
	 * @param point      Location of mouse in client coordinates.
	 *
	 * @return true if message was handled, false to keep searching
	 *         for a handler.
	 */
	virtual bool OnMouseMove(const TPoint &point);
	/**
	 * Mouse down handler.
	 *
	 * @param point      Location of mouse press in client coordinates.
	 *
	 * @return true if message was handled, false to keep searching
	 *         for a handler.
	 */
	virtual bool OnMouseDown(const TPoint &point);
	/**
	 * Initialize the Window. Called by the system only in
	 * Lua initialization.
	 *
	 * When you create your own custom window, this is where you put
	 * your own custom initialization that needs to happen before
	 * children are created. Fundamental window initialization is
	 * handled in every class by this function, so <b>when you
	 * override this function you almost always want to call your
	 * base class to handle base class initialization.</b>
	 */
	virtual void Init(TWindowStyle &style);
	/**
	 * Draw the window. Derived classes will override this
	 * function and provide the draw functionality.
	 */
	virtual void Draw();
	/**
	 * Raw key hit on keyboard.
	 *
	 * @param key    Key pressed on keyboard.
	 * @param flags  TEvent::EKeyFlags mask representing the state of other keys on the keyboard
	 *               when this key was hit.
	 *
	 * @return true if message was handled, false to keep searching
	 *         for a handler.
	 */
	virtual bool OnKeyDown(char key, uint32_t flags);
private:
//	static const int mOffset = 100;
	static const int kNumSprites = 10;
	static const int kBoundary = 2;
	static const int kWidth = 400;
	static const int kHeight = 400;

	uint32_t mHitCount;
	unsigned long mLastUpdate;
	TSpriteRef	mSpriteHolder ;
	TAnimatedSpriteRef	mSprites[kNumSprites];
	TVec2       mVelocity[kNumSprites];

	TAnimatedTextureRef mTexture ;
	TVec2 mLastMouse;

	TLuaParticleSystem	mParticles ;
	TLuaParticleSystem	mParticles2 ;

	TReal mEdgeBounce;
	TReal mDamping;
	TReal mPull;
	TReal mPush;

	SwarmLocus mSwarmLocus ;
};

#endif // SWARM_H_INCLUDED
