#include <iostream>

#include "MayaFileTranslator.h"

#include <maya/MArgList.h>
#include <maya/MFileObject.h>
#include <maya/MItDependencyNodes.h>
#include <maya/MObject.h>
#include <maya/MStatus.h>
#include <maya/MString.h>
#include <maya/MStringArray.h>

#include<maya/MItDag.h>
#include<maya/MFnTransform.h>

#include < maya/MPlug.h >
#include <maya/MObjectArray.h>
#include <maya/MIntArray.h>

#include <maya/MPxFileTranslator.h>



//-------------------------------------------------------------------	MayaFileTranslator
///	\brief  ctor
///
MayaFileTranslator::MayaFileTranslator(void)
{
}

//-------------------------------------------------------------------	~MayaFileTranslator
///	\brief  dtor	
///
MayaFileTranslator::~MayaFileTranslator(void)
{
}


//-------------------------------------------------------------------	reader
///	\brief  This function reads the requested file into maya (import)
///	\param  file	-	info about the file, ie paths, filename etc
///	\param	options	-	mel passes options read from the user
///			interface through to the read function
/// \param	mode	-	ignore for reading
///
MStatus	MayaFileTranslator::reader( const MFileObject& file,
				   const MString& options,
				   FileAccessMode mode)
{
	std::cerr << "Read method as yet unimplimented\n" << std::endl;
	return MS::kFailure;
}

//-------------------------------------------------------------------	writer
///	\brief  Exports the scene data to a text file
///	\param  file	-	info about what to output
/// \param	options	-	a set of output options
/// \param	mode	-	all or selected
///	\return	status code
///
MStatus	MayaFileTranslator::writer( const MFileObject& file,
				   const MString& options,
				   FileAccessMode mode )
{
	MStatus     	stat;

	//	flags that are created by the option script
	//
	const MString	texturePathFlag	("-texturePath");


	//	process the option string passed in from the option script
	//
	if (options.length() > 0) 
	{
        int i, length;
        // Start parsing.
        MStringArray optionList;
        MStringArray theOption;

		//	each option is in the form -
		//	[Option] = [Value];
		//
		//	the options are therefore split by the semi-colons
		//
        options.split(';', optionList);

		//	process each option in the string
		//
		length = optionList.length();
		for( i = 0; i < length; ++i )
		{
            theOption.clear();

			//	split the option using the '=' character. This should make 
			//	theOption[0] hold the option name, and theOption[1] hold the
			//	value of the current option
			//
            optionList[i].split( '=', theOption );

			cout << "option zero is " << theOption[0] << endl;

            if( theOption[0] == texturePathFlag && theOption.length() > 1 ) 
			{
				textureFilePath = theOption[1];

            }
        }
	}
	
	// if this is true, then "Export Selected" was chosen instead of "Export All"
	//
	if( mode == MPxFileTranslator::kExportActiveAccessMode )
	{
		std::cerr << "As yet this does not support export selected\nExporting all instead\n";
	}

	// open the output file
	//
	ofs.open(file.fullName().asChar());

	// make sure it opened ok
	if(!ofs) 
	{
		std::cerr << "[ERROR] Could not open file for writing " << file.fullName().asChar() << std::endl;
	}
	else 
	{
		// Output header data (number of objects and textures)
		outputHeaderData();

		textureList = new MString[textureCount];

		// Output names of textures at the beginning of the file
		outputTextureNames();

		// create iterator to go through all meshes
		MItDag it(MItDag::kDepthFirst,MFn::kMesh); 

		// loop through all nodes
		for( ; !it.isDone(); it.next() )
		{
			// attach function set to the object
			MFnMesh fnMesh(it.item());

			// only want non-history items
			if( !fnMesh.isIntermediateObject() ) {

  				// print mesh name
				// ofs<<"Mesh "<< fnMesh.name().asChar() <<endl;

				outputPositionData( fnMesh );

				outputColorData( fnMesh );

				outputTextureData( fnMesh );

				//ofs << "next" << endl;
				ofs		<< x << " "
						<< y << " "
						<< z << " "
						<< scale << " "
						<< r << " "
						<< g << " "
						<< b << " "
						<< a << " "
						<< getTextureIndex(texture) << endl;

			}
		}
		ofs.close();
	}
	return MS::kSuccess;
}


//-------------------------------------------------------------------	haveReadMethod
///	\brief  returns true if the file translator supports importing 
///	\return	false
///
bool  MayaFileTranslator::haveReadMethod	 () const {
	return false;
}


//-------------------------------------------------------------------	haveWriteMethod
///	\brief  returns true if the file translator supports exporting 
///	\return	true
///
bool MayaFileTranslator::haveWriteMethod	 () const {
	return true;
}


//-------------------------------------------------------------------	defaultExtension
///	\brief  returns the file extension of this format
///
MString MayaFileTranslator::defaultExtension () const {
	return "oes";
}


//-------------------------------------------------------------------	identifyFile
///	\brief  This method is used to check to see if we support the 
///			specified file format. Usually it is sensible to have a 
///			standard identifier at the start of the file that you can
///			check, ie 'MYFILE01'. i don't have that luxury, so instead
///			I am using only a check against the files extension.
///	\param  fileName	-	the name of the file
///	\param	buffer		-	the first few bytes of the file
/// \param	size		-	number of bytes in buffer
///	\return	returns is my file type or is not my file type
///
MPxFileTranslator::MFileKind MayaFileTranslator::identifyFile(	const MFileObject& fileName,
						 const char* buffer,
						 short size) const {
	const char* str = fileName.name().asChar();
	unsigned int len = fileName.name().length();
	if (str[len-3] == 'o' &&
		str[len-2] == 'e' &&
		str[len-1] == 's')
	{
		return kCouldBeMyFileType;
	}
	return kNotMyFileType;
}

//-------------------------------------------------------------------	creator
///	\brief  Maya requires you to provide a static function that 
///			returns the address of a new instance of your node.
///	\return  new file translator node
///
void* MayaFileTranslator::creator() {
	return new MayaFileTranslator;
}




void MayaFileTranslator::outputTextureData(MFnMesh &fnMesh)
{
	// this will hold references to the shaders used on the meshes
	MObjectArray Shaders;

	// this is used to hold indices to the materials returned in the object array
	MIntArray    FaceIndices;

	// get the shaders used by mesh instance 0
	fnMesh.getConnectedShaders(0,Shaders,FaceIndices);

	// Check if just one shader is attached
	if(Shaders.length()!=1) cout << "No shader attached OR too many shaders attached!" << endl;

	// attach a function set to the shading engine
	MFnDependencyNode fn( Shaders[0] );

	// get access to the surfaceShader attribute. This will be connected to
	// lambert , phong nodes etc.
	MPlug sshader = fn.findPlug("surfaceShader");

	// will hold the connections to the surfaceShader attribute
	MPlugArray materials;

	// get the material connected to the surface shader
	sshader.connectedTo(materials,true,false);

	// if we found a material
	if(materials.length())
	{
		MFnDependencyNode fnMat(materials[0].node());

		// get access to the color attribute
		MPlug colorPlug = fnMat.findPlug("color");

		// will hold the connections to the color attribute
		MPlugArray textureNodes;
		colorPlug.connectedTo(textureNodes,true,false);

		// if we found a texture
		if(textureNodes.length()) {

			// Get first texture node
			MFnDependencyNode fnTexFile(textureNodes[0].node());
			
			//ofs << "File name: " << fnTexFile.name() << endl;
			
			// Get connection to file texture name
			MPlug textureFilePlug = fnTexFile.findPlug("ftn");
			MString filepath;

			// Store the full path and name into filepath
			textureFilePlug.getValue(filepath);
			//ofs << "Texture file: " << filepath << endl;

			MString fileNoExt( getFilename(filepath) );

			texture = fileNoExt;
		}

	}
}



void MayaFileTranslator::outputPositionData( MFnMesh &fnMesh )
{
	// Check for multiple parents
	if(fnMesh.parentCount()>1) cout << "Instancing used but not supported!" << endl;
				
	// Output position data of mesh's first parent
	MFnDependencyNode fn(fnMesh.parent(0));

	// write the name of the parent transform
	// ofs << "\t\tparent " << fn.name().asChar() << endl;

	// attach the function set to the object
	MFnTransform fnTransform(fn.object());
	
	double pScale[3];
	fnTransform.getScale(pScale);

	// get the translation values
	MVector translation = fnTransform.getTranslation(MSpace::kTransform);
	
	x = (int) translation.x;
	y = (int) translation.y;
	z = (int) translation.z;
	scale = (float) pScale[0];
}



void MayaFileTranslator::outputColorData( MFnMesh &fnMesh )
{
	MColorArray colors;
	fnMesh.getVertexColors(colors);

	// Check that there is color data, then print colors
	if(colors.length()) {
		
		r = colors[0][0];
		g = colors[0][1];
		b = colors[0][2];
		a = colors[0][3];

	} else {
		r = g = b = a = 1.0;
	}
}

void MayaFileTranslator::outputTextureNames()
{
	// create an iterator to go through all transforms
	MItDependencyNodes it(MFn::kFileTexture); 

	// an array to hold all of the texture nodes
	MObjectArray Files;

	//iterate through all textures
	int i=0;
	while(!it.isDone())
	{
		// attach a dependency node to the file node
		MFnDependencyNode fn(it.item());

		// append the texture to a list of files
		Files.append(it.item());

		// get the attribute for the full texture path
		MPlug ftn = fn.findPlug("ftn");

		// get the filepath from the attribute
		MString filepath;
		ftn.getValue(filepath);

		MString fileNoExt = getFilename( filepath );

		// Output texture name
		ofs << fileNoExt << endl;
		textureList[i] = fileNoExt;

		// get next transform
		it.next(); i++;
	} 

}

MString MayaFileTranslator::getFilename(MString filepath)
{
	// Split the path from the filename
	MStringArray texturePathList;
	filepath.split('/', texturePathList);
	MString fileNoPath( texturePathList[texturePathList.length()-1] );
	//ofs << "Short file name: " << fileNoPath << endl;

	// Split the file extension from the filename
	MStringArray textureFileList;
	fileNoPath.split('.', textureFileList);
	return textureFileList[0];
}

void MayaFileTranslator::outputHeaderData()
{
	// Output:
	// number of objects 
	// number of textures
	// Filepath prefix

	textureCount = objectCount = 0;

	MItDependencyNodes it(MFn::kFileTexture);

	for( ; !it.isDone(); it.next() ) { textureCount++; }
	
	
	// Count objects

	MItDag itMesh(MItDag::kDepthFirst,MFn::kMesh); 
	for( ; !itMesh.isDone(); itMesh.next() ) { 
		 MFnMesh fn(itMesh.item());

		// only want non-history items
		if( !fn.isIntermediateObject() ) {
			MFnDependencyNode fnP(fn.parent(0));
			// Check if transform-name starts with "plane"
			if ( fnP.name().substring(0,4) == MString("plane")) {
				objectCount++;
			}
		}
	 }

	// Output
	ofs << textureCount << " " << objectCount << " " << textureFilePath << endl;
}

int MayaFileTranslator::getTextureIndex(MString texture) {
	for(int i=0; i<textureCount; i++) {
		if (textureList[i] == texture) return i;
	}
	return 0;
}