import maya.cmds as mc
import maya.mel as mm
import os
import struct

# Make a plane without a texture
def makePlane():
    plane = mc.polyPlane(n='plane', w=128, h=128, sx=1, sy=1, ax=[0,0,1])
    planeTranslate = plane[0]+".translate"
    mc.setAttr(planeTranslate, 0+64, 500-64, 0.2,type="double3")

# Opens file dialog to select a texture. Inserts a plane with name and size of the texture.
def makeTexturedPlane():
    textureFile = mc.fileDialog()

    # Continue if file has been chosen
    if textureFile != "":

        # split path and file from string        
        splitPath = os.path.split(textureFile)
        fileName = splitPath[1]
        filePath = splitPath[0]
        fileNameNoExt = fileName[0:-4]
        
        shaderName = fileNameNoExt
        textureNodeName = "file" + fileNameNoExt
        planeName = "plane -" + fileNameNoExt

        width, height = imageSize(textureFile)
        print("Width height ", width , height)
        x = 0 + (width/2)
        y = 0 + (height/2)

        plane = mc.polyPlane(n=planeName, w=width, h=height, sx=1, sy=1, ax=[0,0,1])
        shader = mc.shadingNode('lambert', asShader=True, name=shaderName)
        melCommand = "assignSG " + shader + " " + plane[0]
        mm.eval(melCommand) 

        planeTranslate = plane[0]+".translate"
        mc.setAttr(planeTranslate, x, y, 0.0,type="double3")

        textureNode = mc.shadingNode('file', asTexture=True, name=textureNodeName)
        placementNode = mc.shadingNode('place2dTexture', asUtility=True)

        connectPlacementNodeToTexture(placementNode, textureNode)        
        
        fileAttribute = textureNode+".ftn"
        mc.setAttr( fileAttribute, textureFile, type="string")
        
        mc.connectAttr(textureNode+".outColor", shader+".color", f=True)
        mc.connectAttr(textureNode+".outTransparency", shader+".transparency", f=True)
        
        mc.select(plane)

# Used by makeTexturedPlane 
def connectPlacementNodeToTexture(placementNode, textureNode):  
    mc.connectAttr(placementNode+".coverage", textureNode+".coverage", f=True)
    mc.connectAttr(placementNode+".translateFrame", textureNode+".translateFrame", f=True)
    mc.connectAttr(placementNode+".rotateFrame", textureNode+".rotateFrame", f=True)
    mc.connectAttr(placementNode+".mirrorU", textureNode+".mirrorU", f=True)
    mc.connectAttr(placementNode+".mirrorV", textureNode+".mirrorV", f=True)
    mc.connectAttr(placementNode+".stagger", textureNode+".stagger", f=True)
    mc.connectAttr(placementNode+".wrapU", textureNode+".wrapU", f=True)
    mc.connectAttr(placementNode+".wrapV", textureNode+".wrapV", f=True)
    mc.connectAttr(placementNode+".repeatUV", textureNode+".repeatUV", f=True)
    mc.connectAttr(placementNode+".offset", textureNode+".offset", f=True)
    mc.connectAttr(placementNode+".rotateUV", textureNode+".rotateUV", f=True)
    mc.connectAttr(placementNode+".noiseUV", textureNode+".noiseUV", f=True)
    mc.connectAttr(placementNode+".vertexUvOne", textureNode+".vertexUvOne", f=True)
    mc.connectAttr(placementNode+".vertexUvTwo", textureNode+".vertexUvTwo", f=True)
    mc.connectAttr(placementNode+".vertexUvThree", textureNode+".vertexUvThree", f=True)
    mc.connectAttr(placementNode+".vertexCameraOne", textureNode+".vertexCameraOne", f=True)
    mc.connectAttr(placementNode+".outUV", textureNode+".uv", f=True)
    mc.connectAttr(placementNode+".outUvFilterSize", textureNode+".uvFilterSize", f=True)


# Determine image dimensions  
def imageSize(textureFile):  
    
    textureStream = open(textureFile,"rb")
    fileExt = textureFile[-3:]
    if fileExt == "jpg":
        file, width, height, error = jpgsize(textureStream)
    elif fileExt == "png":
        file, width, height, error = pngsize(textureStream)
    else:
        width, height = 128, 128

    if error != "no error":
            print("Error: ", error)
            
    textureStream.close()
    
    return width, height


# Read header to get dimensions of a png image
# credit python.org
def pngsize(stream):
    (x, y, error) = (None, None, "could not determine PNG size")
    # Dummy read to skip header data
    stream.read(12)
    if stream.read(4) == "IHDR":
        (x, y) = struct.unpack("!LL", stream.read(8))
        error = "no error"
    
    return ("PNG", x, y, error)    


# Read header to get dimensions of a jpg image
# credit python.org
def jpgsize(stream):
	(x, y, error) = (None, None, "could not determine JPEG size")

	# Dummy read to skip header ID
	stream.read(2)
	while True:
		# Extract the segment header.
		(marker, code, length) = struct.unpack("!BBH", stream.read(4))

		# Verify that it's a valid segment.
		if marker != 0xFF:
			# Was it there?
			error = "JPEG marker not found"
			break
		elif code >= 0xC0 and code <= 0xC3:
			# Segments that contain size info
			(y, x) = struct.unpack("!xHH", stream.read(5))
			error = "no error"
			break
		else:
			# Dummy read to skip over data
			stream.read(length - 2)

	return ("JPEG", x, y, error)

# Displays the "Depth Properties" dialog box to change the Z value and scale of selected object(s)
def showTranslateZWindow():
    global translateZSnapping
    if(mc.window('translateZWindow',exists=True))== False:
    	mc.window('translateZWindow', title='Depth Properties', height=200, width=500, sizeable=True )
    	mc.columnLayout(adjustableColumn=True)
    	mc.text(label='', height=7)
    	mc.floatSliderGrp ('translateZSlider', label='Z:', field=True, min=-1500, max=600, fieldMinValue=-20000, fieldMaxValue=1000, value=0, step=10.0, sliderStep=10.0, adjustableColumn3=3, dragCommand=translateZ, changeCommand=translateZtoInt, columnWidth3 = [60,50,400])
    	translateZSnapping = True
    	mc.floatSliderGrp ('speedSlider', label='Speed:', field=True, min=0.1, max=20, value=1, fieldMinValue=0.1, fieldMaxValue=50 ,step=0.1, precision=2,sliderStep=0.1, adjustableColumn3=3, dragCommand=speedObjects, columnWidth3 = [60,50,400])
    	mc.floatSliderGrp ('scaleSlider', label='Scale:', field=True, min=0.1, max=3, value=1, fieldMinValue=0.01, fieldMaxValue=50,step=0.1, precision=2,sliderStep=0.1, adjustableColumn3=3, dragCommand=scaleObject, columnWidth3 = [60,50,400])
    	mc.rowLayout(numberOfColumns=3, columnAlign3=('center', 'center','center'), columnWidth3=(100,100,100), columnAttach3=('left','left','left'), columnOffset3=(20,20,20))
    	mc.checkBox( label='Snap Z slider', align='left', value=True, width=100, onCommand=translateZSnapOn, offCommand=translateZSnapOff )
    	mc.button( label='Get values', width=90, height=20, command=updateDepthSliders)
    	mc.button( label='Reset sliders', width=90, height=20, command=resetDepthSliders)
    	
    mc.showWindow('translateZWindow')
    mc.window('translateZWindow',edit=True,wh=[500,135])

# update z, scale and speed from current selection attributes
def updateDepthSliders(self):
	selection = mc.ls(sl=True, exactType='transform')
	if(selection!=None):
		zAttr = selection[0]+".translateZ"
		zValue = mc.getAttr(zAttr)
		scaleAttr = selection[0]+".scaleX"
		scaleValue = mc.getAttr(scaleAttr)
		speedValue = (800.0-zValue) / 800.0
		mc.floatSliderGrp('translateZSlider',edit=True, value=zValue)
		mc.floatSliderGrp('scaleSlider',edit=True, value=scaleValue)
		mc.floatSliderGrp('speedSlider',edit=True, value=speedValue)

# recreates depth properties window using default settings
def resetDepthSliders(self):
	mc.window('translateZWindow',edit=True,visible=False)
	showTranslateZWindow()


# The function that does the translation for the translate z window
def translateZ(self):
	selection = mc.ls(sl=True, exactType='transform')
	if(selection!=None):
		for i in range(len(selection)):
			objectZattr = selection[i]+".translateZ"
			mc.setAttr(objectZattr, mc.floatSliderGrp('translateZSlider',query=True, value=True ) )


# Convert the floating point value to int if snapping is on
def translateZtoInt(self):
    if translateZSnapping == True:
        zValue=mc.floatSliderGrp('translateZSlider',query=True,value=True)
        mc.floatSliderGrp('translateZSlider',edit=True,value=int(zValue))
        translateZ(self)

# Snap On/Off functions
def translateZSnapOff(self):
    global translateZSnapping
    translateZSnapping = False

def translateZSnapOn(self):
    global translateZSnapping
    translateZSnapping = True


# The function that does the scaling for --z-- window
def scaleObject(self):
	selection = mc.ls(sl=True, exactType='transform')
	if(selection!=None):
		for i in range(len(selection)):
			scaleXAttr = selection[i]+".scaleX"
			scaleYAttr = selection[i]+".scaleY"
			scaleZAttr = selection[i]+".scaleZ"
			newScale = mc.floatSliderGrp('scaleSlider',query=True, value=True )
			mc.setAttr(scaleXAttr, newScale )
			mc.setAttr(scaleYAttr, newScale )
			mc.setAttr(scaleZAttr, newScale )


# Function for speed 
def speedObjects(self):
	scale = mc.floatSliderGrp('speedSlider',query=True, value=True )
	selection = mc.ls(sl=True, exactType='transform')
	
	if(selection!=None):
		for i in range(len(selection)):
			speedObject( selection[i] , scale )

# This function does the actual work for each object
def speedObject( object, scale ):
	z = -((800.0*float(scale))-800.0)

	# create attribute tags
	xAttr = object+".translateX"
	yAttr = object+".translateY"
	zAttr = object+".translateZ"
	scaleXAttr = object+".scaleX"
	scaleYAttr = object+".scaleY"
	scaleZAttr = object+".scaleZ"

	# get current x,y,scale of the object
	xValue = mc.getAttr(xAttr)
	yValue = mc.getAttr(yAttr)
	scaleValue = mc.getAttr(scaleXAttr)

	# convert to untransformed x,y value
	xValue = ((xValue-400)/scaleValue) + 400
	yValue = ((yValue-250)/scaleValue) + 250

	# calculate and set new x, y, scale
	x = ((xValue - 400)*scale) + 400
	y = ((yValue - 250)*scale) + 250
	mc.setAttr(xAttr, x )
	mc.setAttr(yAttr, y )
	mc.setAttr(zAttr, z )
	mc.setAttr(scaleXAttr, scale )
	mc.setAttr(scaleYAttr, scale )
	mc.setAttr(scaleZAttr, scale )

# shows the fit window
def showFitWindow():
    if(mc.window('FitWindow',exists=True))== False:
        mc.window( 'FitWindow', title='Fit image', height=200, width=300, sizeable=False )
        mc.columnLayout()
        mc.text(label='', height=10)
        
        #mc.rowLayout(numberOfColumns=1, columnAlign1=('center'), columnWidth1=(150), columnAttach1=('left'))
        
        mc.rowLayout(numberOfColumns=2, columnAlign2=('center', 'center'), columnWidth2=(150,150), columnAttach2=('left','left'), columnOffset2=(20,20))
        mc.button( label='Fit', width=90, height=30, command=fitImage)
        mc.setParent('..')
        
        mc.text(label='',height=5)
        mc.intFieldGrp( 'fieldFitFrom', label='Fit from:', value1=0, width=300 )
        mc.intFieldGrp( 'fieldFitTo', label='Fit to:', value1=0, width=300 )
        mc.text(label='',height=5)
        mc.radioButtonGrp( 'radioButtonFitDirection', numberOfRadioButtons=2, label='Direction:', labelArray2=['x', 'y'], columnAlign2=('left','left'),select=1,columnWidth3=(100,100,100) )
    mc.showWindow('FitWindow')

# sets the speed of the image from one length of screenspace to another
def fitImage(self):
	fitFrom = mc.intFieldGrp('fieldFitFrom',query=True, value1=True )
	fitTo = mc.intFieldGrp('fieldFitTo',query=True, value1=True )
	fitDirection = mc.radioButtonGrp('radioButtonFitDirection',query=True, select=1 )

	# proceed if fields are not zero
	if(fitFrom != 0 and fitTo != 0):
		
		# if direction is y, convert numbers
		if(fitDirection==2):
			fitFromNumScreens = float(fitFrom) / 500.0
			fitToNumScreens = float(fitTo) / 500.0
			fitFrom = 800.0 * fitFromNumScreens
			fitTo = 800.0 * fitToNumScreens
		
		scale = (float(fitTo)-800.0) / (float(fitFrom)-800.0)
		selection = mc.ls(sl=True, exactType='transform')

		# change speed of selected objects
		if(selection!=None):
			for i in range(len(selection)):
				speedObject( selection[i] , scale )



# General function for turning on/off objects within a certain Z range
def setLayerVisibility(nearLimit, farLimit, toggle):
	selection = mc.ls(exactType='transform')
	if(selection!=None):
		for i in range(len(selection)):
			if selection[i].find('plane') != -1:
				zAttr = selection[i]+".translateZ"
				zValue = mc.getAttr(zAttr)
				if zValue > farLimit and zValue < nearLimit:
					vAttr = selection[i]+".visibility"
					mc.setAttr(vAttr,toggle)        

# shows the animation window
def showAnimationWindow():
    if(mc.window('AniWindow',exists=True))== False:
        mc.window( 'AniWindow', title='Scroll scene', height=200, width=300, sizeable=False )
        mc.columnLayout()
        mc.text(label='', height=10)
        mc.rowLayout(numberOfColumns=2, columnAlign2=('center', 'center'), columnWidth2=(150,150), columnAttach2=('left','left'), columnOffset2=(20,20))
        mc.button( label='Play', width=90, height=30, command=scrollScenePlay)
        mc.button( label='Stop', width=90, height=30, command=scrollSceneStop)
        mc.setParent('..')
        mc.text(label='',height=5)
        mc.intFieldGrp( 'fieldCamXStart', label='Camera X Start:', value1=400, width=300 )
        mc.intFieldGrp( 'fieldCamXEnd', label='Camera X End:', value1=5000, width=300 )
        mc.intFieldGrp( 'fieldCamYStart', label='Camera Y Start:', value1=250, width=300 )
        mc.intFieldGrp( 'fieldCamYEnd', label='Camera Y End:', value1=250, width=300 )
        mc.intFieldGrp( 'fieldSeconds', label='Seconds to scroll:', value1=6, width=300 )
    mc.showWindow('AniWindow')


# will set keyframes using data from animation dialog fields, and play the scroll animation
def scrollScenePlay(self):
	mc.grid(toggle=False)
	xStart = mc.intFieldGrp('fieldCamXStart',query=True, value1=True )
	xEnd = mc.intFieldGrp('fieldCamXEnd',query=True, value1=True )
	yStart = mc.intFieldGrp('fieldCamYStart',query=True, value1=True )
	yEnd = mc.intFieldGrp('fieldCamYEnd',query=True, value1=True )
	secs = mc.intFieldGrp('fieldSeconds',query=True, value1=True )
	endFrame = (secs * 24)+14
	playbackEnd = (endFrame+14)
	mc.playbackOptions( loop='continuous', playbackSpeed=1 )
	mc.playbackOptions( minTime=1, maxTime=playbackEnd, animationEndTime=playbackEnd )

	mc.keyframe('foobyCam.translateX',edit=True,index=(3,3),timeChange=playbackEnd,absolute=True,valueChange=xEnd)
	mc.keyframe('foobyCam.translateY',edit=True,index=(3,3),timeChange=playbackEnd,absolute=True,valueChange=yEnd)
	mc.keyframe('foobyCam.translateZ',edit=True,index=(3,3),absolute=True,timeChange=playbackEnd)

	mc.keyframe('foobyCam.translateX',edit=True,index=(2,2),timeChange=endFrame,absolute=True,valueChange=xEnd)
	mc.keyframe('foobyCam.translateY',edit=True,index=(2,2),timeChange=endFrame,absolute=True,valueChange=yEnd)
	mc.keyframe('foobyCam.translateZ',edit=True,index=(2,2),absolute=True,timeChange=endFrame)	
	
	mc.keyframe('foobyCam.translateX',edit=True,index=(1,1),timeChange=14,absolute=True,valueChange=xStart)
	mc.keyframe('foobyCam.translateY',edit=True,index=(1,1),timeChange=14,absolute=True,valueChange=yStart)
	mc.keyframe('foobyCam.translateZ',edit=True,index=(1,1),absolute=True,timeChange=14)
	
	mc.currentTime(1)
	mc.play()

# stop scrolling animation
def scrollSceneStop(self):
	mc.play(state=False)
	mc.currentTime(1)
